Imports UvARescue.Communication

''' <summary>
''' Sub-class of Agent that adds a CommActor.
''' 
''' This functionality was factored into a separate class since
''' the WSS functionality may be a bit heavy on the CPU. 
''' </summary>
''' <remarks></remarks>
Public Class CommAgent
    Inherits Agent

    Public Sub New(ByVal manifold As Manifold, ByVal name As String, ByVal agentConfig As AgentConfig, ByVal teamConfig As TeamConfig)
        MyBase.New(manifold, name, agentConfig, teamConfig)

        Me._CommActor = Me.CreateCommActor(teamConfig)
        If Not IsNothing(Me._CommActor) Then
            Me.Mount(Me._CommActor)
        End If

    End Sub

    Private _CommActor As CommActor
    Protected ReadOnly Property CommActor() As CommActor
        Get
            Return Me._CommActor
        End Get
    End Property

    Protected Overridable Function CreateCommActor(ByVal teamConfig As TeamConfig) As CommActor
        Return New CommActor(teamConfig, Me.Name, Me.Number, teamConfig.LocalHost)
    End Function

    Protected Friend Overrides Sub OnAgentSpawned()
        MyBase.OnAgentSpawned()
        If Not IsNothing(Me._CommActor) Then
            Me._CommActor.StartComm()
        End If
    End Sub

    Protected Friend Overrides Sub OnAgentStopped()
        If Not IsNothing(Me._CommActor) Then
            Me._CommActor.StopComm()
        End If
        MyBase.OnAgentStopped()
    End Sub

    Public Overrides Sub NotifyPoseEstimateUpdate(ByVal pose As Math.Pose2D)
        MyBase.NotifyPoseEstimateUpdate(pose)
        If Not IsNothing(Me._CommActor) Then
            Me._CommActor.RelayPoseEstimateUpdate(pose)
        End If
    End Sub

End Class
