Imports UvARescue.Math

Public Class OperatorAgent
    Inherits CommAgent

    Public Sub New(ByVal manifold As Manifold, ByVal name As String, ByVal agentConfig As AgentConfig, ByVal teamConfig As TeamConfig)
        MyBase.New(manifold, name, agentConfig, teamConfig)
        Me._Proxies = New Dictionary(Of String, ProxyAgent)
        Me._Connected = New List(Of String)
    End Sub

    Private _Proxies As Dictionary(Of String, ProxyAgent)
    Private _Connected As List(Of String)

    Public Sub RegisterProxy(ByVal proxy As ProxyAgent)
        If Not Me._Proxies.ContainsKey(proxy.Name) Then

            'register proxy
            Me._Proxies.Add(proxy.Name, proxy)
            proxy.SetOperator(Me)

            'it could be that the operator already connected to the alter ego
            If Me._Connected.Contains(proxy.Name) Then
                proxy.NotifyAlterEgoConnected()
            End If

        End If
    End Sub

    Public Sub UnRegisterProxy(ByVal proxy As ProxyAgent)
        If Me._Proxies.ContainsKey(proxy.Name) Then
            proxy.SetOperator(Nothing)
            Me._Proxies.Remove(proxy.Name)
        End If
    End Sub


    Friend Overridable Sub NotifyAgentConnected(ByVal agentName As String)
        If Not Me._Connected.Contains(agentName) Then
            Me._Connected.Add(agentName)
        End If
        If Me._Proxies.ContainsKey(agentName) Then
            Me._Proxies(agentName).NotifyAlterEgoConnected()
        End If
    End Sub
    Friend Overridable Sub NotifyAgentDisconnected(ByVal agentName As String)
        If Me._Connected.Contains(agentName) Then
            Me._Connected.Remove(agentName)
        End If
        If Me._Proxies.ContainsKey(agentName) Then
            Me._Proxies(agentName).NotifyAlterEgoDisconnected()
        End If
    End Sub

    Friend Overridable Sub NotifySignalStrengthReceived(ByVal agentName As String, ByVal pathloss As Double)
        If Me._Proxies.ContainsKey(agentName) Then
            Me._Proxies(agentName).NotifySignalStrengthUpdated(pathloss)
        End If
    End Sub
    Friend Overridable Sub NotifyPoseEstimateReceived(ByVal agentName As String, ByVal pose As Pose2D)
        If Me._Proxies.ContainsKey(agentName) Then

            Dim proxy As ProxyAgent = Me._Proxies(agentName)

            Me.Manifold.AcquireWriterLock()
            Me.Manifold.LocalizeAgent(proxy, Nothing, pose)
            Me.Manifold.ReleaseWriterLock()

        End If
    End Sub

    Friend Overridable Sub RelayUsarSimCommand(ByVal agentName As String, ByVal command As String)
        Me.CommActor.RelayActorCommand(agentName, Command)
    End Sub

    Friend Overridable Sub RelaySyncRequest(ByVal agentName As String)
        Me.CommActor.RelaySyncRequest(agentName)
    End Sub

End Class
