Public Enum BehaviorType

    WaitForStartSignal

    AutonomousExploration

End Enum

Public Class BehaviorControl


    Private _Behaviors As New Dictionary(Of BehaviorType, Behavior)


#Region " Constructor "

    Public Sub New(ByVal agent As Agent, ByVal motionControl As MotionControl)

        Me._Agent = agent
        Me._Manifold = agent.Manifold
        Me._MotionControl = motionControl

        Me._Behaviors.Add(BehaviorType.WaitForStartSignal, New WaitForStartSignal(Me))
        Me._Behaviors.Add(BehaviorType.AutonomousExploration, New AutonomousExploration(Me))

    End Sub

#End Region

#Region " Properties "

    Private _Agent As Agent
    Public ReadOnly Property Agent() As Agent
        Get
            Return Me._Agent
        End Get
    End Property

    Private _Manifold As Manifold
    Public ReadOnly Property Manifold() As Manifold
        Get
            Return Me._Manifold
        End Get
    End Property

    Private _MotionControl As MotionControl
    Public ReadOnly Property MotionControl() As MotionControl
        Get
            Return Me._MotionControl
        End Get
    End Property

#End Region

#Region " Motion Control Wrapper Functions "

    Public Sub ActivateMotion(ByVal motionType As MotionType)
        Me._MotionControl.ActivateMotion(motionType)
    End Sub

    Public Sub DeActivateMotion(ByVal motionType As MotionType)
        Me._MotionControl.DeActivateMotion(motionType)
    End Sub

    Public Sub DeActivateAllMotions()
        Me._MotionControl.DeActivateAllMotions()
    End Sub

#End Region

#Region " Current Behavior Management "

    Protected Friend Overridable Sub NotifySensorUpdate(ByVal sensor As Sensor)
        If Not IsNothing(Me._CurrentBehavior) Then
            Me._CurrentBehavior.NotifySensorUpdate(sensor)
        End If
    End Sub

    Private _CurrentBehavior As Behavior
    Protected ReadOnly Property CurrentBehavior() As Behavior
        Get
            Return Me._CurrentBehavior
        End Get
    End Property

    Public Sub SwitchToBehavior(ByVal behaviorType As BehaviorType)
        If Me._Behaviors.ContainsKey(behaviorType) Then
            Dim behavior As Behavior = Me._Behaviors(behaviorType)
            If Not behavior Is Me._CurrentBehavior Then
                If Not IsNothing(Me._CurrentBehavior) Then
                    Me._CurrentBehavior.DeActivate()
                End If
                If Not IsNothing(behavior) Then
                    behavior.Activate()
                End If
                Me._CurrentBehavior = behavior
            End If
        End If
    End Sub

#End Region

End Class
