Public Enum MotionType

    RandomWalk

    NoMotion

End Enum

Public Class MotionControl


    Private _Motions As New Dictionary(Of MotionType, Motion)


#Region " Constructor "

    Public Sub New(ByVal agent As Agent)

        Me._Agent = agent

        Me._Motions.Add(MotionType.NoMotion, New NoMotion(Me))
        Me._Motions.Add(MotionType.RandomWalk, New RandomWalk(Me))

    End Sub

#End Region

#Region " Properties "

    Private _Agent As Agent
    Public ReadOnly Property Agent() As Agent
        Get
            Return Me._Agent
        End Get
    End Property

#End Region

#Region " Motion Control "

    Protected Friend Overridable Sub NotifySensorUpdate(ByVal sensor As Sensor)
        For Each motion As Motion In Me._ActiveMotions
            motion.NotifySensorUpdate(sensor)
        Next
    End Sub

    Private _ActiveMotions As New List(Of Motion)
    Protected ReadOnly Property ActiveMotions() As IEnumerable(Of Motion)
        Get
            Return Me._ActiveMotions
        End Get
    End Property

    Public Sub ActivateMotion(ByVal motionType As MotionType)
        If Me._Motions.ContainsKey(motionType) Then
            Dim motion As Motion = Me._Motions(motionType)
            If Not Me._ActiveMotions.Contains(motion) Then
                Me._ActiveMotions.Add(motion)
                motion.Activate()
            End If
        End If
    End Sub

    Public Sub DeActivateMotion(ByVal motionType As MotionType)
        If Me._Motions.ContainsKey(motionType) Then
            Dim motion As Motion = Me._Motions(motionType)
            If Me._ActiveMotions.Contains(motion) Then
                motion.DeActivate()
                Me._ActiveMotions.Remove(motion)
            End If
        End If
    End Sub

    Public Sub DeActivateAllMotions()
        For Each motion As Motion In Me._ActiveMotions
            motion.DeActivate()
        Next
        Me._ActiveMotions.Clear()
    End Sub

#End Region

End Class
