Imports System.Math

Public Class RandomWalk
    Inherits Motion

    Protected random As New System.Random(DateTime.Now.Millisecond)

    Protected Enum Movement

        None

        HeadNorth
        TurnEast
        TurnWest
        Retreat

    End Enum


    Public Sub New(ByVal control As MotionControl)
        MyBase.New(control)
    End Sub

    Protected Overrides Sub OnActivated()
        MyBase.OnActivated()
        Me._CurrentMovement = Movement.None
    End Sub

    Protected Overrides Sub OnDeActivated()
        MyBase.OnDeActivated()
        Me._CurrentMovement = Movement.None
    End Sub

    Protected Overrides Sub ProcessSensorUpdate(ByVal sensor As Sensor)
        MyBase.ProcessSensorUpdate(sensor)

        If sensor.SensorType = LaserRangeSensor.SENSORTYPE_RANGESCANNER Then
            Me.ProcessLaserRangeData(DirectCast(sensor, LaserRangeSensor).PeekData)
        ElseIf sensor.SensorType = SonarSensor.SENSORTYPE_SONAR Then
            Me.ProcessSonarData(DirectCast(sensor, SonarSensor).CurrentData)
        ElseIf sensor.SensorType = InsSensor.SENSORTYPE_INS Then
            Me.ProcessInsData(DirectCast(sensor, InsSensor).CurrentData)
        End If

    End Sub

    Private _CurrentMovement As Movement

    Protected Overridable Sub ProcessLaserRangeData(ByVal laser As LaserRangeData)

        Dim fovNorth As Double = 120 / 180 * PI 'frontal 120 degrees
        Dim fovWest As Double = 60 / 180 * PI 'left 60 degrees
        Dim fovEast As Double = 60 / 180 * PI 'right 60 degrees

        Dim minNorth As Double = Double.MaxValue
        Dim avgWest As Double = 0
        Dim avgEast As Double = 0


        'helper vars
        Dim length As Integer = laser.Range.Length - 1
        Dim start As Double
        Dim until As Double
        Dim count As Integer


        'determine min distance in northern direction
        start = (laser.FieldOfView - fovNorth) / 2
        until = laser.FieldOfView - start
        For i As Integer = Max(0, CInt(start / laser.Resolution)) To Min(CInt(until / laser.Resolution), length)
            minNorth = Min(minNorth, laser.Range(i))
        Next

        'determine average dist on the left-front side
        start = (laser.FieldOfView - PI) / 2
        until = start + fovWest
        count = 0
        For i As Integer = Max(0, CInt(start / laser.Resolution)) To Min(CInt(until / laser.Resolution), length)
            avgWest += laser.Range(i)
            count += 1
        Next
        avgWest /= count


        'determine average dist on the right-front side
        until = laser.FieldOfView - (laser.FieldOfView - PI) / 2
        start = until - fovEast
        count = 0
        For i As Integer = Max(0, CInt(start / laser.Resolution)) To Min(CInt(until / laser.Resolution), length)
            avgEast += laser.Range(i)
            count += 1
        Next
        avgEast /= count

        Dim canGoNorth As Boolean = minNorth > 0.55
        Dim westLooksPromising As Boolean
        Dim eastLooksPromising As Boolean

        If avgEast > 0.3 AndAlso avgWest > 0.3 Then
            eastLooksPromising = avgEast > avgWest
            westLooksPromising = avgWest > avgEast
        Else
            eastLooksPromising = avgEast > 0.3
            westLooksPromising = avgWest > 0.3
        End If

        If canGoNorth Then
            Me._CurrentMovement = Movement.HeadNorth
        ElseIf westLooksPromising AndAlso eastLooksPromising Then
            If random.Next(0, 1) = 0 Then
                Me._CurrentMovement = Movement.TurnWest
            Else
                Me._CurrentMovement = Movement.TurnEast
            End If
        ElseIf westLooksPromising Then
            Me._CurrentMovement = Movement.TurnWest
        Else
            Me._CurrentMovement = Movement.TurnEast
        End If

        With Me.Control.Agent
            Select Case Me._CurrentMovement

                Case Movement.None
                    .Halt()

                Case Movement.HeadNorth
                    .Drive(1.0F)

                Case Movement.TurnWest
                    .TurnLeft(0.4F)
                Case Movement.TurnEast
                    .TurnRight(0.4F)

                Case Movement.Retreat
                    .Reverse(0.5F)

            End Select

        End With

    End Sub

    Protected Overridable Sub ProcessSonarData(ByVal sonar As SonarData)
    End Sub

    Protected Overridable Sub ProcessInsData(ByVal ins As InsData)
    End Sub

End Class
