Imports System.Math

''' <summary>
''' Utility class shich can parse several standard logfile formats.
''' </summary>
''' <remarks></remarks>
Public Class LineParsers

    Private Shared Function GetMessageType(ByVal key As String) As MessageType
        Dim msgtype As MessageType

        Select Case key
            Case "SEN"
                msgtype = MessageType.Sensor
            Case "NFO"
                msgtype = MessageType.Info
            Case "STA"
                msgtype = MessageType.Status
            Case "MISSTA"
                msgtype = MessageType.MissionPackageStatus
            Case "GEO"
                msgtype = MessageType.Geometry
            Case "CONF"
                msgtype = MessageType.Configuration
            Case "RES"
                msgtype = MessageType.Response
            Case Else
                Return MessageType.Unknown
        End Select

        Return msgtype
    End Function

    Public Shared Function ParseUsarSimLine(ByVal receiver As Agent, ByVal line As String) As Boolean

        Dim msgtype As MessageType
        Dim type As String = String.Empty
        Dim name As String = String.Empty
        Dim parts As New Specialized.StringCollection

        Dim success As Boolean = False
        If Not String.IsNullOrEmpty(line) Then

            'extract msgtype first
            Dim i As Integer = 0
            Dim j As Integer = line.IndexOf(" ")
            Dim part As String = Nothing
            If j > 0 Then
                part = line.Substring(i, j - i)
            End If
            msgtype = GetMessageType(part)

            If msgtype = MessageType.Unknown Then
                Console.WriteLine("Recieved message of unknown type: " & line)
            End If

            'then extract all curly-braced parts
            i = line.IndexOf("{", j)
            While i >= 0
                j = line.IndexOf("}", i)
                If j >= 0 Then
                    part = line.Substring(i + 1, j - i - 1)
                    parts.Add(part)

                    'we need type and possibly also the name later on in order to know 
                    'which sensor to forward the message to.
                    If String.IsNullOrEmpty(type) AndAlso part.ToUpper.StartsWith("TYPE ") Then
                        type = part.Substring(5).Trim
                    ElseIf String.IsNullOrEmpty(name) AndAlso part.ToUpper.StartsWith("NAME ") Then
                        name = part.Substring(5).Trim

                        'some messages have a different format, then the name is embedded within a part:
                        'e.g.: GEO {Type RangeScanner} {Name Scanner1 Location 0.1440,0.0000,-0.0918 Orientation 0.0000,-0.0000,0.0000 Mount HARD}
                        If name.Contains(" ") Then
                            name = name.Substring(0, name.IndexOf(" "))
                        End If
                    End If

                    i = line.IndexOf("{", j)

                Else
                    'done
                    Exit While

                End If
            End While

            receiver.ReceiveMessage(msgtype, type, name, parts)

            success = True

        End If

        Return success

    End Function

    Public Shared Function ParseMatlabDataLine(ByVal receiver As Agent, ByVal line As String) As Boolean

        'a single line hold a pose estimate and a range scan
        '- the first 3 doubles are the ground-truth pose estimate
        '- the remaining double are the scan which is assumed to be collected over a 180 degree field-of-view



        Dim success As Boolean = False
        If Not String.IsNullOrEmpty(line) Then

            line = line.Replace(vbTab, " ")

            Dim nums() As String = Strings.Split(line, " ")
            Dim pose(3 - 1) As String
            Dim scan(nums.Length - pose.Length - 1) As String

            Array.ConstrainedCopy(nums, 0, pose, 0, 3)
            Array.ConstrainedCopy(nums, pose.Length, scan, 0, scan.Length)
            Array.Reverse(scan)
            'pose(2) = CStr(-Double.Parse(pose(2)))

            'first construct Odometry message
            'SEN {Type INS} {Name INS} {Location 0.80,-7.28,-0.45} {Orientation 0.00,6.28,0.00}
            Dim parts As New Specialized.StringCollection
            parts.Add("Type " & InsSensor.SENSORTYPE_INS)
            parts.Add("Name INS")
            parts.Add(String.Format("Location {0},{1},0.00", pose(0), pose(1)))
            parts.Add(String.Format("Orientation 0.00,0.00,{0}", pose(2)))
            receiver.ReceiveMessage(MessageType.Sensor, InsSensor.SENSORTYPE_INS, "INS", parts)


            'construct similar GroundTruth message
            'SEN {Type GroundTruth} {Name GroundTruth} {Location 0.80,-7.28,-0.45} {Orientation 0.00,6.28,0.00}
            parts.Clear()
            parts.Add("Type " & GroundTruthSensor.SENSORTYPE_GROUNDTRUTH)
            parts.Add("Name GroundTruth")
            parts.Add(String.Format("Location {0},{1},0.00", pose(0), pose(1)))
            parts.Add(String.Format("Orientation 0.00,0.00,{0}", pose(2)))
            receiver.ReceiveMessage(MessageType.Sensor, GroundTruthSensor.SENSORTYPE_GROUNDTRUTH, "GroundTruth", parts)


            'then construct range scan message
            'SEN {Type RangeScanner} {Name Scanner1} {Resolution 0.0174} {FOV 3.1415} {Range <doubles>}
            parts.Clear()
            parts.Add("Type " & LaserRangeSensor.SENSORTYPE_RANGESCANNER)
            parts.Add("Name Scanner1")
            parts.Add("Resolution " + CStr(PI / scan.Length))
            parts.Add("FOV " + CStr(PI))
            parts.Add("Range " + Strings.Join(scan, ","))
            receiver.ReceiveMessage(MessageType.Sensor, LaserRangeSensor.SENSORTYPE_RANGESCANNER, "Scanner1", parts)

            success = True

        End If

        Return success



    End Function

    Public Shared Function ParsePlayerLine(ByVal receiver As Agent, ByVal line As String) As Boolean
        'TBD
    End Function

    Public Shared Function ParseCarmenLine(ByVal receiver As Agent, ByVal line As String) As Boolean
        'TBD
    End Function

    Public Shared Function ParseCognironLine(ByVal receiver As Agent, ByVal line As String) As Boolean
        'TBD
    End Function

End Class

