Imports System.IO

''' <summary>
''' Will use a logfile to obtain sensor readings.
''' </summary>
''' <remarks></remarks>
Public Class LogDriver
    Inherits Driver


    Public Delegate Function ParseDelegate(ByVal reciever As Agent, ByVal line As String) As Boolean


    Public Sub New(ByVal agent As Agent, ByVal logfile As String, ByVal parseHandler As ParseDelegate)
        MyBase.New(agent)
        Me._LogFile = logfile
        Me._Parse = parseHandler
    End Sub


    Private _LogFile As String
    Public ReadOnly Property LogFile() As String
        Get
            Return Me._LogFile
        End Get
    End Property

    Private _Parse As ParseDelegate
    Protected ReadOnly Property Parse() As ParseDelegate
        Get
            Return Me._Parse
        End Get
    End Property

    Protected Overrides Sub Run()

        System.Threading.Thread.CurrentThread.Name = Me.Agent.Name

        'open a reader to the logfile
        Dim reader As StreamReader = File.OpenText(Path.GetFullPath(Me.LogFile))
        Dim line As String

        'a single line will hold a single server message or sensor reading
        While Me.IsRunning AndAlso Not reader.EndOfStream

            Me.CheckForPause()

            line = reader.ReadLine

            'skip commented lines
            If line.StartsWith(My.Settings.CommentChar) Then
                Continue While
            End If

            'skip INFO message, otherwise the Agent code will think that
            'the agent was actually spawned in UsarSim, would cause numerous
            'exceptions
            If line.StartsWith("NFO") Then
                Continue While
            End If

            'forward line the the referenced parsing function
            If Not Me.Parse(Me.Agent, line) Then
                Console.Error.WriteLine("Could not parse line: " & vbNewLine & vbTab & line)
            End If

        End While

    End Sub

End Class
