Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Drawing.Imaging
Imports System.Math
Imports System.Windows.Forms

Imports UvARescue.Math

Public Class AgentsLayer
    Inherits ManifoldLayer

#Region " Constructor / Destructor "

    Public Sub New(ByVal parent As ManifoldImage)
        MyBase.New(parent)
        Me._AgentsColor = Color.Black
        Me.ResetAgentsPen()
    End Sub

    Protected Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then

            Try

                If Not IsNothing(Me._AgentsPen) Then
                    Me._AgentsPen.Dispose()
                    Me._AgentsPen = Nothing
                End If

            Catch ex As Exception
                Console.Error.WriteLine(ex.ToString)

            End Try

        End If

        MyBase.Dispose(disposing)

    End Sub

#End Region

#Region " Maintain Information "

    Private _AgentsColor As Color
    Private _AgentsPen As Pen
    Public Property AgentsColor() As Color
        Get
            Return Me._AgentsColor
        End Get
        Set(ByVal value As Color)
            If Not value = Me._AgentsColor Then
                Me._AgentsColor = value
                Me.ResetAgentsPen()
            End If
        End Set
    End Property
    Protected ReadOnly Property AgentsPen() As Pen
        Get
            Return Me._AgentsPen
        End Get
    End Property
    Private Sub ResetAgentsPen()
        If Not IsNothing(Me._AgentsPen) Then
            Me._AgentsPen.Dispose()
        End If
        Me._AgentsPen = New Pen(Me._AgentsColor)
    End Sub


    Private poses As New Dictionary(Of Guid, Pose2D)

    Private sizes As New Dictionary(Of Guid, Size)
    Private names As New Dictionary(Of Guid, String)

    Public Overrides Sub RenderAgent(ByVal agent As Agent, ByVal pose As Math.Pose2D)
        MyBase.RenderAgent(agent, pose)

        If Not Me.poses.ContainsKey(agent.UniqueID) Then
            Me.poses.Add(agent.UniqueID, pose)
            Me.sizes.Add(agent.UniqueID, agent.Size)
            Me.names.Add(agent.UniqueID, agent.Name)
        Else
            'no need to update sizes and names, only the pose is dynamic
            Me.poses(agent.UniqueID) = pose
        End If

    End Sub

    Friend Overrides Sub NotifyImageReset()
        MyBase.NotifyImageReset()

        Me.poses.Clear()
        Me.sizes.Clear()
        Me.names.Clear()

    End Sub

#End Region

#Region " Rendering "

    Public Overrides Sub Draw(ByVal g As Graphics, ByVal renderingMode As ManifoldRenderingMode)
        MyBase.Draw(g, renderingMode)

        If Not IsNothing(Me.poses) AndAlso Me.poses.Count > 0 Then

            Dim path As New GraphicsPath
            Dim pose As Pose2D
            Dim size As Size

            For Each key As Guid In Me.poses.Keys
                pose = Me.poses(key)
                size = Me.sizes(key)

                'construct ellipse centered on pose
                path.StartFigure()
                path.AddEllipse(New Rectangle(New Point(CInt(pose.X - size.Width / 2), CInt(pose.Y - size.Height / 2)), size))
                path.CloseFigure()

                'construct line indicating the rotation
                Dim len As Single = Max(size.Height, size.Width)
                Dim dx As Single = CType(Cos(pose.Rotation) * len, Single)
                Dim dy As Single = CType(Sin(pose.Rotation) * len, Single)

                path.StartFigure()
                path.AddLine(CType(pose.X, Single), CType(pose.Y, Single), CType(pose.X + dx, Single), CType(pose.Y + dy, Single))
                path.CloseFigure()

                'also paint the agent's name just outside the circle's radius
                Me.DrawReadableString(g, renderingMode, New PointF(CType(pose.X + 0.5F * Max(size.Height, size.Width), Single), CType(pose.Y + 0.5F * Max(size.Height, size.Width), Single)), Me.names(key))

            Next

            'transform from woorld coords to page coords
            path.Transform(Me.Parent.TransformationMatrix)

            g.DrawPath(Me._AgentsPen, path)

        End If

    End Sub

#End Region

End Class
