Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Drawing.Imaging
Imports System.Math
Imports System.Windows.Forms

Imports UvARescue.Math

Public Class FreeSpaceLayer
    Inherits BufferedLayer

    Public Sub New(ByVal parent As ManifoldImage)
        MyBase.New(parent)
        'Me._FreeSpaceAlpha = CInt(255 * 0.2) '20%
        Me._FreeSpaceAlpha = CInt(255) '100%
        Me._FreeSpaceColor = Color.White
        Me.ResetFreeSpacePen()
    End Sub

    Protected Overrides Sub Dispose(ByVal disposing As Boolean)
        If Not IsNothing(Me._FreeSpacePen) Then
            Me._FreeSpacePen.Dispose()
            Me._FreeSpacePen = Nothing
        End If
        MyBase.Dispose(disposing)
    End Sub

    Private _FreeSpaceAlpha As Integer
    Private _FreeSpaceColor As Color
    Private _FreeSpacePen As Pen
    Public Property FreeSpaceAlpha() As Integer
        Get
            Return Me._FreeSpaceAlpha
        End Get
        Set(ByVal value As Integer)
            If Not value = Me._FreeSpaceAlpha Then
                Me._FreeSpaceAlpha = value
                Me.ResetFreeSpacePen()
            End If
        End Set
    End Property
    Public Property FreeSpaceColor() As Color
        Get
            Return Me._FreeSpaceColor
        End Get
        Set(ByVal value As Color)
            If Not value = Me._FreeSpaceColor Then
                Me._FreeSpaceColor = value
                Me.ResetFreeSpacePen()
            End If
        End Set
    End Property
    Protected ReadOnly Property FreeSpacePen() As Pen
        Get
            Return Me._FreeSpacePen
        End Get
    End Property
    Private Sub ResetFreeSpacePen()
        If Not IsNothing(Me._FreeSpacePen) Then
            Me._FreeSpacePen.Dispose()
        End If
        Me._FreeSpacePen = New Pen(Color.FromArgb(Me._FreeSpaceAlpha, Me._FreeSpaceColor))
    End Sub

    Public Overrides Sub RenderPatch(ByVal patch As Patch, ByVal origin As Math.Vector2, ByVal points() As Math.Vector2)
        MyBase.RenderPatch(patch, origin, points)

        'create path in manifold-ooords (= world coords)
        Dim freeSpace As New GraphicsPath
        For Each point As Vector2 In points
            freeSpace.AddLine(CType(origin.X, Single), CType(origin.Y, Single), CType(point.X, Single), CType(point.Y, Single))
        Next

        Me.Parent.EnsureMinimumImageBounds(freeSpace.GetBounds(Me.Parent.TransformationMatrix))

        'plot the path into the graphics buffer
        Me.Gfx.DrawPath(Me._FreeSpacePen, freeSpace)

    End Sub

End Class
