Imports System.Drawing
Imports System.IO
Imports MathNet.Numerics
Imports MathNet.SignalProcessing.DataSources


Public Enum CameraImageType As Byte
    Raw
    Jpeg
End Enum

Public Class CameraData
    Implements ISensorData

#Region " Constructor "

    Public Sub New()
    End Sub

#End Region

#Region " Properties "

    Private _CameraImageType As CameraImageType
    Public ReadOnly Property CameraImageType() As Byte
        Get
            Return _CameraImageType
        End Get
    End Property

    Private _Width As Integer = 0
    Public ReadOnly Property Width() As Integer
        Get
            Return _Width
        End Get
    End Property

    Private _Height As Integer = 0
    Public ReadOnly Property Height() As Integer
        Get
            Return _Height
        End Get
    End Property

    Private _RawData As Byte() = Nothing
    Public ReadOnly Property RawData() As Byte()
        Get
            Return _RawData
        End Get
    End Property

    Private _Bitmap As Bitmap = Nothing
    Public ReadOnly Property Bitmap() As Bitmap
        Get
            Return Me._Bitmap
        End Get
    End Property

    Private _Sequence As Integer = 0
    Public ReadOnly Property Sequence() As Integer
        Get
            Return _Sequence
        End Get
    End Property


#End Region

#Region " Load "

    Public Sub Load(ByVal bytes As Byte(), ByVal spawnNumber As Integer, ByVal multiviewMode As Byte, ByVal sequenceNumber As Integer)

        Me._Sequence = sequenceNumber

        If bytes(0) = 0 Then
            Me._CameraImageType = UvARescue.Agent.CameraImageType.Raw
        Else
            Me._CameraImageType = UvARescue.Agent.CameraImageType.Jpeg
        End If

        ' convert 4 bytes to Integer
        Dim length As Integer = CInt(bytes(1) * 2 ^ 24) + CInt(bytes(2) * 2 ^ 16) + CInt(bytes(3) * 2 ^ 8) + CInt(bytes(4) * 2 ^ 0)

        Select Case Me._CameraImageType

            Case UvARescue.Agent.CameraImageType.Raw

                Me._Width = CInt(bytes(5) * 2 ^ 8) + CInt(bytes(6))
                Me._Height = CInt(bytes(7) * 2 ^ 8) + CInt(bytes(8))

                'copy raw data RGB(1 byte + 1 byte + 1 byte )
                ReDim Me._RawData(length - 5)
                For i As Integer = 0 To Me._RawData.Length - 1
                    Me._RawData(i) = bytes(i + 9)
                Next

                'get Multiview subview 
                Me._RawData = GetSubView(Me._RawData, spawnNumber)

            Case Else
                'jpeg

                Dim jpeg(length - 1) As Byte
                For i As Integer = 0 To jpeg.Length - 1
                    jpeg(i) = bytes(i + 5)
                Next

                'if Multiview is not in use
                If multiviewMode < 2 Then
                    Me._Bitmap = DirectCast(New Bitmap(New System.IO.MemoryStream(jpeg)).Clone, Bitmap)

                Else
                    'get Multiview subview 

                    Dim subView As Bitmap
                    subView = DirectCast(New Bitmap(New System.IO.MemoryStream(jpeg)).Clone, Bitmap)

                    Me._Bitmap = GetSubView(subView, spawnNumber, multiviewMode)

                End If

                Me._Width = Me._Bitmap.Width
                Me._Height = Me._Bitmap.Height

                Dim R(Me._Width * Me._Height) As Complex
                Dim G(Me._Width * Me._Height) As Complex
                Dim B(Me._Width * Me._Height) As Complex

                Try
                    BitmapConverter.ReadChannel(Me._Bitmap, R, G, B)
                Catch ex As Exception
                    Console.WriteLine("Error occurred while trying to read RGB channels.")
                    Console.WriteLine(ex.Message & vbNewLine & ex.StackTrace)
                End Try

                ReDim Me._RawData(R.Length * 3)

                Dim index As Integer = 0
                For i As Integer = 0 To (R.Length) - 1
                    _RawData(index) = CByte(R(i).Real * 256)
                    _RawData(index + 1) = CByte(G(i).Real * 256)
                    _RawData(index + 2) = CByte(B(i).Real * 256)
                    index += 3
                Next

        End Select

    End Sub

    'subview getter function for Raw data
    Private Function GetSubView(ByVal rawData As Byte(), ByVal spawnNumber As Integer) As Byte()

        'TODO: Raw subview getter function should be implemented here

        Dim subViewRaw As Byte() = Nothing
        Return subViewRaw
    End Function

    'subview getter function for Jpeg data
    Private Function GetSubView(ByVal bMap As Bitmap, ByVal spawnNumber As Integer, ByVal multiviewMode As Byte) As Bitmap

        Dim fullView As Bitmap = New Bitmap(bMap)

        'saving cameradata to file
        'Dim pathName, imageFile As String
        'pathName = Path.Combine(My.Application.Info.DirectoryPath, "Images")
        'imageFile = Path.Combine(pathName, String.Concat(CStr(spawnNumber), "_FULL.jpg"))
        'Dim fileInfo As New FileInfo(imageFile)
        'If Not fileInfo.Directory.Exists Then
        '   FileInfo.Directory.Create()
        'End If
        'fullView.Save(imageFile, System.Drawing.Imaging.ImageFormat.Jpeg)

        Dim row, column, x, y, width, height As Integer

        row = (spawnNumber - 1) Mod multiviewMode
        column = (spawnNumber - 1) \ multiviewMode

        width = fullView.Width \ multiviewMode
        height = fullView.Height \ multiviewMode

        x = row * width
        y = column * height

        Dim cloneRect As New Drawing.Rectangle(x, y, width, height)
        Dim format As Imaging.PixelFormat = fullView.PixelFormat
        Dim subView As Bitmap = fullView.Clone(cloneRect, format)

        'saving cameradata to file
        'imageFile = Path.Combine(pathName, String.Concat(CStr(spawnNumber), "_SUBVIEW.jpg"))
        'subView.Save(imageFile, System.Drawing.Imaging.ImageFormat.Jpeg)

        Return subView

    End Function

    Public Sub Load(ByVal msg As System.Collections.Specialized.StringDictionary) Implements ISensorData.Load
    End Sub

#End Region

End Class
