<Serializable()> _
Public Class Matrix

#Region " Constructors / Factory Methods "

    'copy constructor
    Public Sub New(ByVal source As Matrix)
        Me.New(source.Rows, source.Cols)
        For row As Integer = 0 To Me.Rows - 1
            For col As Integer = 0 To Me.Cols - 1
                Me(row, col) = source(row, col)
            Next
        Next
    End Sub

    Public Sub New(ByVal rows As Integer, ByVal cols As Integer)
        Me._rows = rows
        Me._cols = cols
        ReDim Me._m(rows - 1, cols - 1)
    End Sub

    Public Sub New(ByVal size As Integer)
        Me.New(size, size)
    End Sub

    Public Sub LoadIdentity()
        For row As Integer = 0 To Me.Rows - 1
            For col As Integer = 0 To Me.Cols - 1
                If row = col Then
                    Me(row, col) = 1
                Else
                    Me(row, col) = 0
                End If
            Next
        Next
    End Sub

    Public Shared Function Identity(ByVal size As Integer) As Matrix
        Dim matrix As New Matrix(size, size)
        matrix.LoadIdentity()
        Return matrix
    End Function

    Public Overloads Shared Function Zeros(ByVal size As Integer) As Matrix
        Return Zeros(size, size)
    End Function
    Public Overloads Shared Function Zeros(ByVal rows As Integer, ByVal cols As Integer) As Matrix
        Dim matrix As New Matrix(rows, cols)
        For row As Integer = 0 To rows - 1
            For col As Integer = 0 To cols - 1
                matrix(row, col) = 0
            Next
        Next
        Return matrix
    End Function

    Public Overloads Shared Function Ones(ByVal size As Integer) As Matrix
        Return Ones(size, size)
    End Function
    Public Overloads Shared Function Ones(ByVal rows As Integer, ByVal cols As Integer) As Matrix
        Dim matrix As New Matrix(rows, cols)
        For row As Integer = 0 To rows - 1
            For col As Integer = 0 To cols - 1
                matrix(row, col) = 1
            Next
        Next
        Return matrix
    End Function

#End Region

#Region " Data "

    Private _m(,) As Double
    Default Public Overloads Property Item(ByVal row As Integer, ByVal col As Integer) As Double
        Get
            Return Me._m(row, col)
        End Get
        Set(ByVal value As Double)
            Me._m(row, col) = value
        End Set
    End Property

    Private _rows As Integer
    Public ReadOnly Property Rows() As Integer
        Get
            Return Me._rows
        End Get
    End Property

    Private _cols As Integer
    Public ReadOnly Property Cols() As Integer
        Get
            Return Me._cols
        End Get
    End Property

    Public Overrides Function ToString() As String

        Dim builder As New Text.StringBuilder
        builder.AppendFormat("Matrix of size {0}x{1}", Me.Rows, Me.Cols).AppendLine()
        builder.Append(" [ ")
        For i As Integer = 0 To Me.Rows - 1
            For j As Integer = 0 To Me.Cols - 1
                builder.Append(Me(i, j) & " ")
            Next
            If i < Me.Rows - 1 Then
                builder.AppendLine()
            Else
                builder.Append("]").AppendLine()
            End If
        Next

        Return builder.ToString()

    End Function

#End Region

#Region " Operators "

    Public Overloads Function Negative() As Matrix
        Dim result As New Matrix(Me.Rows, Me.Cols)
        For row As Integer = 0 To Me.Rows - 1
            For col As Integer = 0 To Me.Cols - 1
                result(row, col) = -Me(row, col)
            Next
        Next
        Return result
    End Function
    Public Overloads Shared Function Negative(ByVal matrix As Matrix) As Matrix
        Return matrix.Negative
    End Function

    Public Overloads Function Add(ByVal right As Matrix) As Matrix
        If right.Rows <> Me.Rows OrElse right.Cols <> Me.Cols Then _
            Throw New InvalidOperationException("Matrixes must be of equal dimensions")
        Dim result As New Matrix(Me.Rows, Me.Cols)
        For row As Integer = 0 To Me.Rows - 1
            For col As Integer = 0 To Me.Cols - 1
                result(row, col) = Me(row, col) + right(row, col)
            Next
        Next
        Return result
    End Function
    Public Overloads Shared Function Add(ByVal left As Matrix, ByVal right As Matrix) As Matrix
        Return left.Add(right)
    End Function

    Public Overloads Function Subtract(ByVal right As Matrix) As Matrix
        If right.Rows <> Me.Rows OrElse right.Cols <> Me.Cols Then _
            Throw New InvalidOperationException("Matrixes must be of equal dimensions")
        Dim result As New Matrix(Me.Rows, Me.Cols)
        For row As Integer = 0 To Me.Rows - 1
            For col As Integer = 0 To Me.Cols - 1
                result(row, col) = Me(row, col) - right(row, col)
            Next
        Next
        Return result
    End Function
    Public Overloads Shared Function Subtract(ByVal left As Matrix, ByVal right As Matrix) As Matrix
        Return left.Subtract(right)
    End Function

    Public Overloads Function Multiply(ByVal factor As Double) As Matrix
        Dim result As New Matrix(Me.Rows, Me.Cols)
        For row As Integer = 0 To Me.Rows - 1
            For col As Integer = 0 To Me.Cols - 1
                result(row, col) = factor * Me(row, col)
            Next
        Next
        Return result
    End Function
    Public Overloads Shared Function Multiply(ByVal factor As Double, ByVal matrix As Matrix) As Matrix
        Return matrix.Multiply(factor)
    End Function

    Public Overloads Function Multiply(ByVal right As Matrix) As Matrix
        If Me.Cols <> right.Rows Then _
            Throw New InvalidOperationException("Inner Matrix dimensions must agree")
        Dim result As New Matrix(Me.Rows, right.Cols)
        Dim sum As Double
        For row As Integer = 0 To Me.Rows - 1
            For col As Integer = 0 To right.Cols - 1
                sum = 0
                For idx As Integer = 0 To Me.Cols - 1
                    sum += Me(row, idx) * right(idx, col)
                Next
                result(row, col) = sum
            Next
        Next
        Return result
    End Function
    Public Overloads Shared Function Multiply(ByVal left As Matrix, ByVal right As Matrix) As Matrix
        Return left.Multiply(right)
    End Function

    Public Overloads Function Multiply(ByVal vector As Vector) As Vector
        If Me.Cols <> vector.Length Then _
            Throw New InvalidOperationException("Dimensions must agree")
        Dim result As New Vector(Me.Rows)
        Dim sum As Double
        For row As Integer = 0 To Me.Rows - 1
            sum = 0
            For col As Integer = 0 To Me.Cols - 1
                sum += Me(row, col) * vector(col)
            Next
            result(row) = sum
        Next
        Return result
    End Function
    Public Overloads Shared Function Multiply(ByVal left As Matrix, ByVal right As Vector) As Vector
        Return left.Multiply(right)
    End Function

    Public Overridable Overloads Function Determinant() As Double
        Throw New NotSupportedException("Determinant computation not supported on Matrix")
    End Function
    Public Overloads Shared Function Determinant(ByVal matrix As Matrix) As Double
        Return matrix.Determinant
    End Function

    Public Overloads Function Trace() As Double
        If Me.Rows <> Me.Cols Then _
            Throw New InvalidOperationException("Cannot compute Trace for non-square matrixes")
        Dim result As Double = 0
        For idx As Integer = 0 To Me.Rows - 1
            result += Me(idx, idx)
        Next
        Return result
    End Function
    Public Overloads Shared Function Trace(ByVal matrix As Matrix) As Double
        Return matrix.Trace
    End Function

    Public Overloads Function Transpose() As Matrix
        Dim result As New Matrix(Me.Cols, Me.Rows)
        For row As Integer = 0 To Me.Rows - 1
            For col As Integer = 0 To Me.Cols - 1
                result(col, row) = Me(row, col)
            Next
        Next
        Return result
    End Function
    Public Overloads Shared Function Transpose(ByVal matrix As Matrix) As Matrix
        Return matrix.Transpose()
    End Function



    Public Overloads Shared Operator -(ByVal right As Matrix) As Matrix
        Return right.Negative
    End Operator

    Public Overloads Shared Operator +(ByVal left As Matrix, ByVal right As Matrix) As Matrix
        Return left.Add(right)
    End Operator
    Public Overloads Shared Operator -(ByVal left As Matrix, ByVal right As Matrix) As Matrix
        Return left.Subtract(right)
    End Operator

    Public Overloads Shared Operator *(ByVal factor As Double, ByVal matrix As Matrix) As Matrix
        Return matrix.Multiply(factor)
    End Operator
    Public Overloads Shared Operator *(ByVal matrix As Matrix, ByVal factor As Double) As Matrix
        Return matrix.Multiply(factor)
    End Operator
    Public Overloads Shared Operator *(ByVal matrix As Matrix, ByVal vector As Vector) As Vector
        Return matrix.Multiply(vector)
    End Operator
    Public Overloads Shared Operator *(ByVal left As Matrix, ByVal right As Matrix) As Matrix
        Return left.Multiply(right)
    End Operator

#End Region

End Class
