Imports UvARescue.Agent
Imports UvARescue.Math

Imports System.Math


Public Class IdcScanMatcher
    Inherits IcpScanMatcher

    Protected Overrides Function MatchPoints(ByVal points1() As Vector2, ByVal points2() As Vector2, ByVal odometry As Pose2D, ByVal dangle As Double, ByVal filter1() As Boolean, ByVal filter2() As Boolean) As MatchResult

        Dim startTime As DateTime = Now

        Dim rawdpose As Pose2D = odometry
        Dim curdpose As Pose2D = rawdpose
        Dim curpoints2() As Vector2 = Nothing
        Dim curpairs(,) As Integer = Nothing

        Dim results As New Queue(Of Double)
        Dim nconverged As Integer = 0

        Dim uavg As New Vector2
        Dim sumerror As Double = 0.0
        Dim sumangcov As Double = 0.0


        Dim n As Integer = 0
        For n = 1 To MAX_ITERATIONS

            'recompute relative coords for range-scan 2 given the new dpose estimate
            curpoints2 = Me.ToLocal(points2, curdpose)
            'find all correlated point-pairs
            curpairs = Me.CorrelatePoints(points1, curpoints2, filter1, filter2)

            If curpairs.Length > 0 Then

                'these vars will hold summaries for a single iteration
                Dim sumpoint1 As New Vector2
                Dim sumpoint2 As New Vector2

                For p As Integer = 0 To curpairs.GetLength(0) - 1
                    sumpoint1 += points1(curpairs(p, 0))
                    sumpoint2 += curpoints2(curpairs(p, 1))
                Next

                Dim u1 As Vector2 = sumpoint1 / curpairs.GetLength(0)
                Dim u2 As Vector2 = sumpoint2 / curpairs.GetLength(0)
                uavg = (u1 + u2) / 2

                sumpoint1 = New Vector2
                sumpoint2 = New Vector2
                sumerror = 0.0
                sumangcov = 0.0

                For p As Integer = 0 To curpairs.GetLength(0) - 1

                    Dim point1 As Vector2 = points1(curpairs(p, 0))
                    Dim point2 As Vector2 = curpoints2(curpairs(p, 1))

                    sumpoint1 += New Vector2( _
                        (point1(0) - u1(0)) * (point2(0) - u2(0)), _
                        (point1(1) - u1(1)) * (point2(1) - u2(1)))
                    sumpoint2 += New Vector2( _
                        (point1(0) - u1(0)) * (point2(1) - u2(0)), _
                        (point1(1) - u1(1)) * (point2(0) - u2(1)))

                    Dim dpoint As Vector2 = point1 - point2
                    Dim mpoint As Vector2 = (point1 + point2) / 2

                    sumerror += dpoint * dpoint
                    sumangcov += mpoint * mpoint

                Next

                'update odometry estimate
                Dim rotest As Double = Atan2(sumpoint2(0) - sumpoint2(1), sumpoint1(0) + sumpoint1(1))
                Dim rotmx As New TMatrix2D(rotest)
                Dim transest As Vector2 = u2 - rotmx * u1

                curdpose = New Pose2D(curdpose.Position - transest, curdpose.Rotation - rotest)
                rotmx = New TMatrix2D(-rotest)
                curdpose = rotmx * curdpose


                'check for convergence
                Dim converged As Boolean = False
                For Each result As Double In results
                    If sumerror = 0 Then
                        converged = converged OrElse (result < ERROR_THRESHOLD)
                    Else
                        converged = converged OrElse (Abs(result / sumerror - 1) < ERROR_THRESHOLD)
                    End If
                Next

                results.Enqueue(sumerror)
                While results.Count > RESULT_MAXCONSIDERED
                    results.Dequeue()
                End While

                If converged Then
                    nconverged += 1
                End If

            End If

            If nconverged >= RESULT_CONVERGED Then
                Exit For
            End If

        Next


        Dim mcov As New Matrix3
        Dim num As Integer = 0

        If Not IsNothing(curpairs) Then

            num = curpairs.GetLength(0)

            Dim cov As New MathNet.Numerics.LinearAlgebra.Matrix(3, 3)
            cov(0, 0) = num
            cov(0, 1) = 0
            cov(0, 2) = -num * uavg(1)
            cov(1, 0) = 0
            cov(1, 1) = num
            cov(1, 2) = num * uavg(0)
            cov(2, 0) = -num * uavg(1)
            cov(2, 1) = num * uavg(0)
            cov(2, 2) = sumangcov

            cov = (sumerror / (2 * num - 3)) * cov.Inverse

            mcov(0, 0) = CType(cov(0, 0), Single)
            mcov(0, 1) = CType(cov(0, 1), Single)
            mcov(0, 2) = CType(cov(0, 2), Single)
            mcov(1, 0) = CType(cov(1, 0), Single)
            mcov(1, 1) = CType(cov(1, 1), Single)
            mcov(1, 2) = CType(cov(1, 2), Single)
            mcov(2, 0) = CType(cov(2, 0), Single)
            mcov(2, 1) = CType(cov(2, 1), Single)
            mcov(2, 2) = CType(cov(2, 2), Single)

        End If

        Dim untilTime As DateTime = Now
        Dim duration As TimeSpan = untilTime - startTime

        Return New MatchResult(rawdpose, curdpose, mcov, n, num, CInt(duration.TotalMilliseconds), Me.HasConverged(points1.Length, num, n))

    End Function

End Class
