Imports UvARescue.Agent
Imports UvARescue.Math

Imports System.Math


Public Class MbIcpScanMatcher
    Inherits IcpScanMatcher

    'In Minguez' code the 'magical' L is set to 3
    Protected Const LSQ As Single = 3.0 * 3.0

    Protected Overrides Function MatchPoints(ByVal points1() As Vector2, ByVal points2() As Vector2, ByVal odometry As Pose2D, ByVal dangle As Double, ByVal filter1() As Boolean, ByVal filter2() As Boolean) As MatchResult

        Dim startTime As DateTime = Now

        Dim rawdpose As Pose2D = odometry
        Dim curdpose As Pose2D = rawdpose
        Dim curpoints2() As Vector2 = Nothing
        Dim curpairs(,) As Integer = Nothing

        Dim uavg As New Vector2
        Dim sumerror As Double = 0.0
        Dim sumangcov As Double = 0.0

        Dim converged As Boolean = False


        Dim n As Integer = 0
        For n = 1 To MAX_ITERATIONS

            'recompute relative coords for range-scan 2 given the new dpose estimate
            curpoints2 = Me.ToLocal(points2, curdpose)
            'find all correlated point-pairs
            curpairs = Me.CorrelatePoints(points1, curpoints2, filter1, filter2)

            If curpairs.Length > 0 Then

                'these vars will hold summaries for a single iteration
                Dim sumpoint1 As New Vector2
                Dim sumpoint2 As New Vector2


                Dim A As New MathNet.Numerics.LinearAlgebra.Matrix(3, 3)
                Dim b As New MathNet.Numerics.LinearAlgebra.Matrix(3, 1)

                sumerror = 0
                sumangcov = 0.0

                For p As Integer = 0 To curpairs.GetLength(0) - 1

                    Dim point1 As Vector2 = points1(curpairs(p, 0))
                    Dim point2 As Vector2 = curpoints2(curpairs(p, 1))
                    Dim dpoint As Vector2 = point2 - point1
                    Dim mpoint As Vector2 = (point1 + point2) / 2

                    sumpoint1 += points1(curpairs(p, 0))
                    sumpoint2 += curpoints2(curpairs(p, 1))
                    sumerror += dpoint * dpoint
                    sumangcov += mpoint * mpoint

                    'MbICP needs the points in meters
                    Dim pix As Double = point1.X / 1000
                    Dim piy As Double = point1.Y / 1000
                    Dim cix As Double = point2.X / 1000
                    Dim ciy As Double = point2.Y / 1000

                    Dim ki As Double = pix * pix + piy * piy + LSQ
                    Dim cxpxPcypy As Double = cix * pix + ciy * piy
                    Dim cxpyMcypx As Double = cix * piy - ciy * pix

                    A(0, 0) += 1.0 - piy * piy / ki
                    A(0, 1) += pix * piy / ki
                    A(0, 2) += -ciy + piy / ki * cxpxPcypy
                    A(1, 1) += 1.0 - pix * pix / ki
                    A(1, 2) += cix - pix / ki * cxpxPcypy
                    A(2, 2) += cix * cix + ciy * ciy - cxpxPcypy * cxpxPcypy / ki

                    b(0, 0) += cix - pix - piy / ki * cxpyMcypx
                    b(1, 0) += ciy - piy + pix / ki * cxpyMcypx
                    b(2, 0) += (cxpxPcypy / ki - 1.0) * cxpyMcypx

                Next

                'Complete the A-matrix by assigning the symmetric portions of it
                A(1, 0) = A(0, 1)
                A(2, 0) = A(0, 2)
                A(2, 1) = A(1, 2)

                'compute the transformation that minimizes 
                A = A.Inverse
                A.Multiply(-1)

                Dim qmin As MathNet.Numerics.LinearAlgebra.Matrix = A * b
                Dim qminpose As New Pose2D(CType(qmin(0, 0), Single), CType(qmin(1, 0), Single), CType(qmin(2, 0), Single))

                'check for convergence
                Dim threshold As Double = 0.0001
                converged = True
                converged = converged AndAlso qminpose.X < threshold
                converged = converged AndAlso qminpose.Y < threshold
                converged = converged AndAlso qminpose.Rotation < threshold

                Dim tmpdpose As New Pose2D(0, 0, 0)
                tmpdpose.X = CType(curdpose.X * Cos(qminpose.Rotation) - curdpose.Y * Sin(qminpose.Rotation) + qminpose.X, Single)
                tmpdpose.Y = CType(curdpose.X * Sin(qminpose.Rotation) + curdpose.Y * Cos(qminpose.Rotation) + qminpose.Y, Single)
                tmpdpose.Rotation = CType(curdpose.Rotation + qminpose.Rotation, Single)

                curdpose = tmpdpose

            End If

            If converged Then
                Exit For
            End If

        Next

        Dim mcov As New Matrix3
        Dim num As Integer = 0

        If Not IsNothing(curpairs) Then

            num = curpairs.GetLength(0)

            Dim cov As New MathNet.Numerics.LinearAlgebra.Matrix(3, 3)
            cov(0, 0) = num
            cov(0, 1) = 0
            cov(0, 2) = -num * uavg(1)
            cov(1, 0) = 0
            cov(1, 1) = num
            cov(1, 2) = num * uavg(0)
            cov(2, 0) = -num * uavg(1)
            cov(2, 1) = num * uavg(0)
            cov(2, 2) = sumangcov

            cov = (sumerror / (2 * num - 3)) * cov.Inverse

            mcov(0, 0) = CType(cov(0, 0), Single)
            mcov(0, 1) = CType(cov(0, 1), Single)
            mcov(0, 2) = CType(cov(0, 2), Single)
            mcov(1, 0) = CType(cov(1, 0), Single)
            mcov(1, 1) = CType(cov(1, 1), Single)
            mcov(1, 2) = CType(cov(1, 2), Single)
            mcov(2, 0) = CType(cov(2, 0), Single)
            mcov(2, 1) = CType(cov(2, 1), Single)
            mcov(2, 2) = CType(cov(2, 2), Single)

        End If

        curdpose.X *= 1000
        curdpose.Y *= 1000

        Dim untilTime As DateTime = Now
        Dim duration As TimeSpan = untilTime - startTime

        Return New MatchResult(rawdpose, curdpose, mcov, n, num, CInt(duration.TotalMilliseconds), Me.HasConverged(points1.Length, num, n))

    End Function

    Protected Overrides Function ComputeSquaredDistance(ByVal point1 As Math.Vector2, ByVal point2 As Math.Vector2) As Double
        Dim dx As Double = point2.X - point1.X
        Dim dy As Double = point2.Y - point1.Y
        Dim tmp As Double = dx * point1.Y - dy * point1.X
        Return dx ^ 2 + dy ^ 2 - (tmp ^ 2 / (point1.Y ^ 2 + point1.X ^ 2 + LSQ))
    End Function

End Class
