Imports System.Net
Imports System.Net.Sockets
Imports System.Text

Public Class TcpConnection

    Protected client As New TcpClient
    Protected stream As NetworkStream = Nothing


#Region " Client - Connect / Disconnect "

    Public ReadOnly Property IsConnected() As Boolean
        Get
            Return Me.client.Connected
        End Get
    End Property

    Public Sub Connect(ByVal host As String, ByVal port As Integer)
        If Me.IsConnected Then Throw New InvalidOperationException("Agent already connected")
        Me.client.Connect(host, port)
        Me.OpenStream()
    End Sub

    Public Sub Disconnect()
        If Not Me.IsConnected Then Throw New InvalidOperationException("Agent not connected")
        Me.CloseStream()
        Me.client.Close()
    End Sub

#End Region

#Region " Stream - Open / Close "

    Public ReadOnly Property DataAvailable() As Boolean
        Get
            If Not Me.IsConnected Then Throw New InvalidOperationException("You should connect first")
            Return Me.stream.DataAvailable
        End Get
    End Property

    Protected Sub OpenStream()
        Me.stream = Me.client.GetStream()
        If Not stream.CanRead Then Throw New Exception("Stream is not readable")
        If Not stream.CanWrite Then Throw New Exception("Stream is not writable")
    End Sub

    Protected Sub CloseStream()
        Me.stream.Flush()
        Me.stream.Close()
        Me.stream.Dispose()
        Me.stream = Nothing
    End Sub

#End Region

#Region " Send and Recieve Strings "

    ''' <summary>
    ''' a string-based (ASCII) message sending format is assumed.
    ''' </summary>
    ''' <param name="message"></param>
    ''' <remarks></remarks>
    Public Sub Send(ByVal message As String)
        If Not Me.IsConnected Then Throw New InvalidOperationException("You should connect first")

        'convert to bytes and send it
        Dim bytes() As Byte = Encoding.ASCII.GetBytes(message)
        Me.stream.Write(bytes, 0, bytes.Length)
        Me.stream.Flush()

    End Sub

    Public Function Recieve(ByVal maxLength As Integer) As String
        If Not Me.IsConnected Then Throw New InvalidOperationException("You should connect first")

        'recieve any inbound messages
        If stream.DataAvailable Then

            'load data into buffer, max 4K per iteration
            Dim buffer(maxLength) As Byte

            'length will tell how many bytes were actually retrieved
            Dim length As Integer = stream.Read(buffer, 0, buffer.Length)

            Return Encoding.ASCII.GetString(buffer, 0, length)

        End If

        Return String.Empty

    End Function

#End Region

End Class
