Imports UvARescue.Agent
Imports UvARescue.Math
Imports UvARescue.Slam

''' <summary>
''' Base class used for USAR Agents.
''' </summary>
''' <remarks></remarks>
Public Class UsarAgent
    Inherits CommAgent
    'Inherits SmartAgent

    Public Sub New(ByVal manifold As Manifold, ByVal name As String, ByVal agentConfig As AgentConfig, ByVal teamConfig As TeamConfig)
        MyBase.New(manifold, name, agentConfig, teamConfig)

        ''NOTE: these are mounted by Agent already
        'Me.Mount(New InsSensor) '2007 format
        'Me.Mount(New OdometrySensor)

        If agentConfig.RobotModel.ToLower = "talon" Then
            'the talon has no room for a SICK, so we equipped it with a Hokuyo
            Me.Mount(New LaserRangeSensor("Scanner1", LaserRangeDeviceType.Hokuyo, 0.2F, 3.8F, 0.38F, -0.134285569F))
        Else
            'default to a SICK for all other robots
            Me.Mount(New LaserRangeSensor("Scanner1", LaserRangeDeviceType.SickLMS, 0.2F, 19.8F, 0.0F, 0.0F))
        End If

        Me.Mount(New CameraSensor(agentConfig))
        Me.Mount(New RfidSensor)
        Me.Mount(New SonarSensor)
        Me.Mount(New TouchSensor)
        Me.Mount(New VictimSensor) ' 2007 format

        'for backwards compatibility with logfiles from 2006, we also mount the now 
        'obsolete sensors.
        Me.Mount(New InuSensor) '2006 format, becamse InsSensor in 2007
        Me.Mount(New VictimRfidSensor) '2006 format, becamse VictimSensor in 2007

    End Sub

    Private counter As Integer = 0
    Public Overrides Sub NotifyPoseEstimateUpdate(ByVal pose As Math.Pose2D)
        MyBase.NotifyPoseEstimateUpdate(pose)

        'keep track of the number of localization updates
        Me.counter += 1
        Console.WriteLine(String.Format("[{0}]- {1:d3}: current pose = {2}", Me.Name, counter, pose.ToString))

    End Sub

End Class
